@@ -41,12 +41,14 @@ os.environ.setdefault('EDITOR', 'vi')
 
 from cliff.app import App
 from cliff.commandmanager import CommandManager, LOG
+from requests.packages import urllib3
 
 import slapos.version
 
 # silence messages like 'Starting connection' that are logged with INFO
 urllib3_logger = logging.getLogger('requests.packages.urllib3')
 urllib3_logger.setLevel(logging.WARNING)
+urllib3.disable_warnings()
 
 
 class SlapOSCommandManager(CommandManager):
@@ -131,9 +133,6 @@ class SlapOSApp(App):
 
     log = logging.getLogger('slapos')
 
-    CONSOLE_MESSAGE_FORMAT = '%(message)s'
-    LOG_FILE_MESSAGE_FORMAT = '[%(asctime)s] %(levelname)-8s %(message)s'
-
     def __init__(self):
         super(SlapOSApp, self).__init__(
             description='SlapOS client %s' % slapos.version.version,
@@ -177,6 +176,18 @@ class SlapOSApp(App):
             default=None,
             help='Specify a file to log output (default: console only)',
         )
+        parser.add_argument(
+            '--log-color',
+            action='store_true',
+            help='Colored log output in console (stripped if redirected)',
+            default=True,
+        )
+        parser.add_argument(
+            '--log-time',
+            action='store_false',
+            default=True,
+            help='Include timestamp in console log',
+        )
         parser.add_argument(
             '-h', '--help',
             action=SlapOSHelpAction,
@@ -198,6 +209,54 @@ class SlapOSApp(App):
         if self.options.verbose_level > 2:
             self.log.debug('clean_up %s', cmd.__class__.__name__)
 
+    @property
+    def CONSOLE_MESSAGE_FORMAT(self):
+        fmt = []
+        if self.options.log_time and not self.options.log_color:
+            fmt.append('[%(asctime)s]')
+        if not self.options.log_color:
+            fmt.append('%(levelname)s')
+        fmt.append('%(message)s')
+        return ' '.join(fmt)
+
+    @property
+    def LOG_FILE_MESSAGE_FORMAT(self):
+        return '[%(asctime)s] %(levelname)-8s %(message)s'
+
+    def configure_logging(self):
+        """Create logging handlers for any log output.
+        """
+        root_logger = logging.getLogger('')
+        root_logger.setLevel(logging.DEBUG)
+
+        # Set up logging to a file
+        if self.options.log_file:
+            file_handler = logging.FileHandler(
+                filename=self.options.log_file,
+            )
+            formatter = logging.Formatter(self.LOG_FILE_MESSAGE_FORMAT)
+            file_handler.setFormatter(formatter)
+            root_logger.addHandler(file_handler)
+
+        # Always send higher-level messages to the console via stderr
+        if self.options.log_color:
+            import coloredlogs
+            console = coloredlogs.ColoredStreamHandler(show_name=True,     # logger name (slapos) and PID
+                                                       show_severity=True,
+                                                       show_timestamps=self.options.log_time,
+                                                       show_hostname=False)
+        else:
+            console = logging.StreamHandler(self.stderr)
+        console_level = {0: logging.WARNING,
+                         1: logging.INFO,
+                         2: logging.DEBUG,
+                         }.get(self.options.verbose_level, logging.DEBUG)
+        console.setLevel(console_level)
+        formatter = logging.Formatter(self.CONSOLE_MESSAGE_FORMAT)
+        console.setFormatter(formatter)
+        root_logger.addHandler(console)
+        return
+
     def run(self, argv):
         # same as cliff.app.App.run except that it won't re-raise
         # a logged exception, and doesn't use --debug
