@@ -48,19 +48,54 @@ from slapos.grid import utils  # for methods that could be mocked, access them t
 from slapos.slap.slap import NotFoundError
 from slapos.grid.svcbackend import getSupervisorRPC
 from slapos.grid.exception import (BuildoutFailedError, WrongPermissionError,
-                                   PathDoesNotExistError)
+                                   PathDoesNotExistError, DiskSpaceError)
 from slapos.grid.networkcache import download_network_cached, upload_network_cached
+from slapos.human import bytes2human
 
 
 WATCHDOG_MARK = '-on-watch'
 
 REQUIRED_COMPUTER_PARTITION_PERMISSION = 0o750
 
+CP_STORAGE_FOLDER_NAME = 'DATA'
+
 # XXX not very clean. this is changed when testing
 PROGRAM_PARTITION_TEMPLATE = pkg_resources.resource_stream(__name__,
             'templates/program_partition_supervisord.conf.in').read()
 
 
+def free_space(path, fn):
+  while True:
+    try:
+      disk = os.statvfs(path)
+      return fn(disk)
+    except OSError:
+      pass
+    if os.sep not in path:
+      break
+    path = os.path.split(path)[0]
+
+
+def free_space_root(path):
+  """
+  Returns free space available to the root user, in bytes.
+
+  A non-existent path can be provided, and the ancestors
+  will be queried instead.
+  """
+  return free_space(path, lambda d: d.bsize * d.f_bfree)
+
+
+def free_space_nonroot(path):
+  """
+  Returns free space available to non-root users, in bytes.
+
+  A non-existent path can be provided, and the ancestors
+  will be queried instead.
+  """
+  return free_space(path, lambda d: d.f_bsize * d.f_bavail)
+
+
 class Software(object):
   """This class is responsible for installing a software release"""
 
@@ -73,7 +108,8 @@ class Software(object):
                download_binary_cache_url=None, upload_binary_cache_url=None,
                download_binary_dir_url=None, upload_binary_dir_url=None,
                download_from_binary_cache_url_blacklist=None,
-               upload_to_binary_cache_url_blacklist=None):
+               upload_to_binary_cache_url_blacklist=None,
+               software_min_free_space=None):
     """Initialisation of class parameters
     """
 
@@ -106,6 +142,17 @@ class Software(object):
         download_from_binary_cache_url_blacklist
     self.upload_to_binary_cache_url_blacklist = \
         upload_to_binary_cache_url_blacklist
+    self.software_min_free_space = software_min_free_space
+
+  def check_free_space(self):
+    required = self.software_min_free_space
+    available = free_space_nonroot(self.software_path)
+
+    if available < required:
+      msg = "Not enough space for {path}: available {available}, required {required} (option 'software_min_free_space')"
+      raise DiskSpaceError(msg.format(path=self.software_path,
+                                      available=bytes2human(available),
+                                      required=bytes2human(required)))
 
   def install(self):
     """ Fetches binary cache if possible.
@@ -113,6 +160,9 @@ class Software(object):
     """
     self.logger.info("Installing software release %s..." % self.url)
     cache_dir = tempfile.mkdtemp()
+
+    self.check_free_space()
+
     try:
       tarpath = os.path.join(cache_dir, self.software_url_hash)
       # Check if we can download from cache
@@ -293,6 +343,9 @@ class Partition(object):
                logger,
                certificate_repository_path=None,
                retention_delay='0',
+               instance_min_free_space=None,
+               instance_storage_home='',
+               ipv4_global_network='',
                ):
     """Initialisation of class parameters"""
     self.buildout = buildout
@@ -309,6 +362,8 @@ class Partition(object):
     self.partition_id = partition_id
     self.server_url = server_url
     self.software_release_url = software_release_url
+    self.instance_storage_home = instance_storage_home
+    self.ipv4_global_network = ipv4_global_network
 
     self.key_file = ''
     self.cert_file = ''
@@ -333,6 +388,19 @@ class Partition(object):
         self.instance_path, self.retention_lock_date_filename
     )
 
+    self.instance_min_free_space = instance_min_free_space
+
+
+  def check_free_space(self):
+    required = self.instance_min_free_space
+    available = free_space_nonroot(self.instance_path)
+
+    if available < required:
+      msg = "Not enough space for {path}: available {available}, required {required} (option 'instance_min_free_space')"
+      raise DiskSpaceError(msg.format(path=self.instance_path,
+                                      available=bytes2human(available),
+                                      required=bytes2human(required)))
+
   def _updateCertificate(self):
     try:
       partition_certificate = self.computer_partition.getCertificate()
@@ -397,6 +465,9 @@ class Partition(object):
     """
     self.logger.info("Installing Computer Partition %s..."
         % self.computer_partition.getId())
+
+    self.check_free_space()
+
     # Checks existence and permissions of Partition directory
     # Note : Partitions have to be created and configured before running slapgrid
     if not os.path.isdir(self.instance_path):
@@ -447,6 +518,8 @@ class Partition(object):
             'software_release_url': self.software_release_url,
             'key_file': self.key_file,
             'cert_file': self.cert_file,
+            'storage_home': self.instance_storage_home,
+            'global_ipv4_network_prefix': self.ipv4_global_network,
         }
     open(config_location, 'w').write(buildout_text)
     os.chmod(config_location, 0o640)
@@ -465,7 +538,7 @@ class Partition(object):
     os.chown(config_location, -1, int(gid))
     if len(bootstrap_candidate_list) == 0:
       buildout_binary = os.path.join(self.software_path, 'bin', 'buildout')
-      self.logger.warning("Falling back to default buildout %r" %
+      self.logger.info("Falling back to default buildout %r" %
         buildout_binary)
     else:
       if len(bootstrap_candidate_list) != 1:
@@ -505,6 +578,7 @@ class Partition(object):
                                 ['buildout:bin-directory=%s' %
                                     os.path.join(self.instance_path, 'sbin')])
       buildout_binary = os.path.join(self.instance_path, 'sbin', 'buildout')
+
     # Launches buildout
     utils.launchBuildout(path=self.instance_path,
                          buildout_binary=buildout_binary,
@@ -618,12 +692,22 @@ class Partition(object):
       sr_symlink = os.path.join(self.instance_path, 'software_release')
       if os.path.islink(sr_symlink):
         os.unlink(sr_symlink)
+      data_base_link = os.path.join(self.instance_path, CP_STORAGE_FOLDER_NAME)
+      if self.instance_storage_home and os.path.exists(data_base_link) and \
+                                os.path.isdir(data_base_link):
+        for filename in os.listdir(data_base_link):
+          data_symlink = os.path.join(data_base_link, filename)
+          partition_data_path = os.path.join(self.instance_storage_home,
+                                                    filename, self.partition_id)
+          if os.path.lexists(data_symlink):
+            os.unlink(data_symlink)
+          if os.path.exists(partition_data_path):
+            self.cleanupFolder(partition_data_path)
 
-      for root, dirs, file_list in os.walk(self.instance_path):
-        for directory in dirs:
-          shutil.rmtree(os.path.join(self.instance_path, directory))
-        for file in file_list:
-          os.remove(os.path.join(self.instance_path, file))
+      self.cleanupFolder(self.instance_path)
+      
+      # Cleanup all Data storage location of this partition
+      
 
       if os.path.exists(self.supervisord_partition_configuration_path):
         os.remove(self.supervisord_partition_configuration_path)
@@ -633,6 +717,15 @@ class Partition(object):
 
     return True
 
+  def cleanupFolder(self, folder_path):
+    """Delete all files and folders in a specified directory
+    """
+    for root, dirs, file_list in os.walk(folder_path):
+      for directory in dirs:
+        shutil.rmtree(os.path.join(folder_path, directory))
+      for file in file_list:
+        os.remove(os.path.join(folder_path, file))
+
   def fetchInformations(self):
     """Fetch usage informations with buildout, returns it.
     """
@@ -660,7 +753,16 @@ class Partition(object):
         self.logger.warning('Problem while stopping process %r, will try later' % gname)
       else:
         self.logger.info('Stopped %r' % gname)
-      supervisor.removeProcessGroup(gname)
+      for i in xrange(0, 10):
+        # Some process may be still running, be nice and wait for them to be stopped.
+        try:
+          supervisor.removeProcessGroup(gname)
+          break
+        except:
+          if i == 9:
+            raise
+          time.sleep(1)
+
       self.logger.info('Removed %r' % gname)
 
     for gname in changed:
