@@ -37,7 +37,6 @@ import subprocess
 import sys
 import tempfile
 import time
-import stat
 import traceback
 import warnings
 import logging
@@ -50,12 +49,15 @@ from lxml import etree
 
 from slapos.slap.slap import NotFoundError
 from slapos.slap.slap import ServerError
-from slapos.util import mkdir_p, chownDirectory
+from slapos.util import mkdir_p, chownDirectory, string_to_boolean
 from slapos.grid.exception import BuildoutFailedError
 from slapos.grid.SlapObject import Software, Partition
-from slapos.grid.svcbackend import launchSupervisord
-from slapos.grid.utils import (md5digest, createPrivateDirectory, dropPrivileges,
-                               SlapPopen, updateFile)
+from slapos.grid.svcbackend import (launchSupervisord,
+                                    createSupervisordConfiguration,
+                                    _getSupervisordConfigurationDirectory,
+                                    _getSupervisordSocketPath)
+from slapos.grid.utils import (md5digest, dropPrivileges, SlapPopen)
+from slapos.human import human2bytes
 import slapos.slap
 
 
@@ -70,12 +72,12 @@ SLAPGRID_FAIL = 1
 SLAPGRID_PROMISE_FAIL = 2
 PROMISE_TIMEOUT = 3
 
-# XXX hardcoded watchdog_path
-WATCHDOG_PATH = '/opt/slapos/bin/slapos-watchdog'
-
 COMPUTER_PARTITION_TIMESTAMP_FILENAME = '.timestamp'
 COMPUTER_PARTITION_LATEST_BANG_TIMESTAMP_FILENAME = '.slapos_latest_bang_timestamp'
 
+# XXX hardcoded watchdog_path
+WATCHDOG_PATH = '/opt/slapos/bin/slapos-watchdog'
+
 
 class _formatXMLError(Exception):
   pass
@@ -148,15 +150,6 @@ def merged_options(args, configp):
   if options.get('all'):
     options['develop'] = True
 
-  # Supervisord configuration location
-  if not options.get('supervisord_configuration_path'):
-    options['supervisord_configuration_path'] = \
-      os.path.join(options['instance_root'], 'etc', 'supervisord.conf')
-  # Supervisord socket
-  if not options.get('supervisord_socket'):
-    options['supervisord_socket'] = \
-      os.path.join(options['instance_root'], 'supervisord.socket')
-
   # Parse cache / binary cache options
   # Backward compatibility about "binary-cache-url-blacklist" deprecated option
   if (options.get("binary-cache-url-blacklist") and not
@@ -201,12 +194,13 @@ def create_slapgrid_object(options, logger):
     ]
 
   op = options
+  software_min_free_space = human2bytes(op.get('software_min_free_space', '200M'))
+  instance_min_free_space = human2bytes(op.get('instance_min_free_space', '100M'))
+
   return Slapgrid(software_root=op['software_root'],
                   instance_root=op['instance_root'],
                   master_url=op['master_url'],
                   computer_id=op['computer_id'],
-                  supervisord_socket=op['supervisord_socket'],
-                  supervisord_configuration_path=op['supervisord_configuration_path'],
                   buildout=op.get('buildout'),
                   logger=logger,
                   maximum_periodicity = op.get('maximum_periodicity', 86400),
@@ -231,11 +225,16 @@ def create_slapgrid_object(options, logger):
                   shacache_key_file=op.get('shacache-key-file'),
                   shadir_cert_file=op.get('shadir-cert-file'),
                   shadir_key_file=op.get('shadir-key-file'),
+                  forbid_supervisord_automatic_launch=string_to_boolean(op.get('forbid_supervisord_automatic_launch', 'false')),
                   develop=op.get('develop', False),
                   # Try to fetch from deprecated argument
                   software_release_filter_list=op.get('only-sr', op.get('only_sr')),
                   # Try to fetch from deprecated argument
-                  computer_partition_filter_list=op.get('only-cp', op.get('only_cp')))
+                  computer_partition_filter_list=op.get('only-cp', op.get('only_cp')),
+                  software_min_free_space=software_min_free_space,
+                  instance_min_free_space=instance_min_free_space,
+                  instance_storage_home=op.get('instance_storage_home'),
+                  ipv4_global_network=op.get('ipv4_global_network'))
 
 
 def check_required_only_partitions(existing, required):
@@ -261,8 +260,6 @@ class Slapgrid(object):
                instance_root,
                master_url,
                computer_id,
-               supervisord_socket,
-               supervisord_configuration_path,
                buildout,
                logger,
                maximum_periodicity=86400,
@@ -285,9 +282,14 @@ class Slapgrid(object):
                shacache_key_file=None,
                shadir_cert_file=None,
                shadir_key_file=None,
+               forbid_supervisord_automatic_launch=False,
                develop=False,
                software_release_filter_list=None,
                computer_partition_filter_list=None,
+               software_min_free_space=None,
+               instance_min_free_space=None,
+               instance_storage_home=None,
+               ipv4_global_network=None,
                ):
     """Makes easy initialisation of class parameters"""
     # Parses arguments
@@ -295,8 +297,7 @@ class Slapgrid(object):
     self.instance_root = os.path.abspath(instance_root)
     self.master_url = master_url
     self.computer_id = computer_id
-    self.supervisord_socket = supervisord_socket
-    self.supervisord_configuration_path = supervisord_configuration_path
+    self.supervisord_socket = _getSupervisordSocketPath(instance_root)
     self.key_file = key_file
     self.cert_file = cert_file
     self.master_ca_file = master_ca_file
@@ -317,6 +318,7 @@ class Slapgrid(object):
     self.shacache_key_file = shacache_key_file
     self.shadir_cert_file = shadir_cert_file
     self.shadir_key_file = shadir_key_file
+    self.forbid_supervisord_automatic_launch = forbid_supervisord_automatic_launch
     self.logger = logger
     # Creates objects from slap module
     self.slap = slapos.slap.slap()
@@ -324,8 +326,6 @@ class Slapgrid(object):
         cert_file=self.cert_file, master_ca_file=self.master_ca_file)
     self.computer = self.slap.registerComputer(self.computer_id)
     # Defines all needed paths
-    self.supervisord_configuration_directory = \
-        os.path.join(self.instance_root, 'etc', 'supervisord.conf.d')
     self.buildout = buildout
     self.promise_timeout = promise_timeout
     self.develop = develop
@@ -339,8 +339,18 @@ class Slapgrid(object):
       self.computer_partition_filter_list = \
           computer_partition_filter_list.split(",")
     self.maximum_periodicity = maximum_periodicity
+    self.software_min_free_space = software_min_free_space
+    self.instance_min_free_space = instance_min_free_space
+    if instance_storage_home:
+      self.instance_storage_home = os.path.abspath(instance_storage_home)
+    else:
+      self.instance_storage_home = ""
+    if ipv4_global_network:
+      self.ipv4_global_network = ipv4_global_network
+    else:
+      self.ipv4_global_network= ""
 
-  def getWatchdogLine(self):
+  def _getWatchdogLine(self):
     invocation_list = [WATCHDOG_PATH]
     invocation_list.append("--master-url '%s' " % self.master_url)
     if self.certificate_repository_path:
@@ -357,42 +367,12 @@ class Slapgrid(object):
     # Checks for software_root and instance_root existence
     if not os.path.isdir(self.software_root):
       raise OSError('%s does not exist.' % self.software_root)
-    if not os.path.isdir(self.instance_root):
-      raise OSError('%s does not exist.' % self.instance_root)
-    # Creates everything needed
 
-    # Create directory accessible for the instances.
-    var_directory = os.path.join(self.instance_root, 'var')
-    if not os.path.isdir(var_directory):
-      os.mkdir(var_directory)
+    createSupervisordConfiguration(self.instance_root, self._getWatchdogLine())
 
-    os.chmod(var_directory, stat.S_IRWXU | stat.S_IROTH | stat.S_IXOTH | \
-                            stat.S_IRGRP | stat.S_IXGRP )
-
-    mkdir_p(os.path.join(self.instance_root, 'var'), 0o755)
-
-    # Creates instance_root structure
-    createPrivateDirectory(os.path.join(self.instance_root, 'var', 'log'))
-    createPrivateDirectory(os.path.join(self.instance_root, 'var', 'run'))
-
-    createPrivateDirectory(os.path.join(self.instance_root, 'etc'))
-    createPrivateDirectory(self.supervisord_configuration_directory)
-
-    # Creates supervisord configuration
-    updateFile(self.supervisord_configuration_path,
-      pkg_resources.resource_stream(__name__,
-        'templates/supervisord.conf.in').read() % {
-            'supervisord_configuration_directory': self.supervisord_configuration_directory,
-            'supervisord_socket': os.path.abspath(self.supervisord_socket),
-            'supervisord_loglevel': 'info',
-            'supervisord_logfile': os.path.abspath(os.path.join(self.instance_root, 'var', 'log', 'supervisord.log')),
-            'supervisord_logfile_maxbytes': '50MB',
-            'supervisord_nodaemon': 'false',
-            'supervisord_pidfile': os.path.abspath(os.path.join(self.instance_root, 'var', 'run', 'supervisord.pid')),
-            'supervisord_logfile_backups': '10',
-            'watchdog_command': self.getWatchdogLine(),
-        }
-    )
+  def _launchSupervisord(self):
+    if not self.forbid_supervisord_automatic_launch:
+      launchSupervisord(instance_root=self.instance_root, logger=self.logger)
 
   def getComputerPartitionList(self):
     try:
@@ -433,7 +413,8 @@ class Slapgrid(object):
             shacache_cert_file=self.shacache_cert_file,
             shacache_key_file=self.shacache_key_file,
             shadir_cert_file=self.shadir_cert_file,
-            shadir_key_file=self.shadir_key_file)
+            shadir_key_file=self.shadir_key_file,
+            software_min_free_space=self.software_min_free_space)
         if state == 'available':
           completed_tag = os.path.join(software_path, '.completed')
           if (self.develop or (not os.path.exists(completed_tag) and
@@ -490,11 +471,6 @@ class Slapgrid(object):
       return SLAPGRID_FAIL
     return SLAPGRID_SUCCESS
 
-  def _launchSupervisord(self):
-    launchSupervisord(self.supervisord_socket,
-                      self.supervisord_configuration_path,
-                      logger=self.logger)
-
   def _checkPromises(self, computer_partition):
     self.logger.info("Checking promises...")
     instance_path = os.path.join(self.instance_root, computer_partition.getId())
@@ -517,7 +493,7 @@ class Slapgrid(object):
         command = [os.path.join(promise_dir, promise)]
 
         promise = os.path.basename(command[0])
-        self.logger.info("Checking promise %r.", promise)
+        self.logger.info("Checking promise '%s'.", promise)
 
         process_handler = subprocess.Popen(command,
                                            preexec_fn=lambda: dropPrivileges(uid, gid, logger=self.logger),
@@ -530,18 +506,27 @@ class Slapgrid(object):
         process_handler.stdin.close()
         process_handler.stdin = None
 
-        time.sleep(self.promise_timeout)
-
-        if process_handler.poll() is None:
-          process_handler.terminate()
-          raise Slapgrid.PromiseError("The promise %r timed out" % promise)
-        elif process_handler.poll() != 0:
-          stderr = process_handler.communicate()[1]
-          if stderr is None:
-            stderr = 'No error output from %r.' % promise
+        # Check if the promise finished every tenth of second,
+        # but timeout after promise_timeout.
+        sleep_time = 0.1
+        increment_limit = int(self.promise_timeout / sleep_time)
+        for current_increment in range(0, increment_limit):
+          if process_handler.poll() is None:
+            time.sleep(sleep_time)
+            continue
+          if process_handler.poll() == 0:
+            # Success!
+            break
           else:
-            stderr = 'Promise %r:' % promise + stderr
-          raise Slapgrid.PromiseError(stderr)
+            stderr = process_handler.communicate()[1]
+            if stderr is None:
+              stderr = "No error output from '%s'." % promise
+            else:
+              stderr = "Promise '%s':" % promise + stderr
+            raise Slapgrid.PromiseError(stderr)
+        else:
+          process_handler.terminate()
+          raise Slapgrid.PromiseError("The promise '%s' timed out" % promise)
 
     if not promise_present:
       self.logger.info("No promise.")
@@ -658,7 +643,7 @@ class Slapgrid(object):
         software_path=software_path,
         instance_path=instance_path,
         supervisord_partition_configuration_path=os.path.join(
-          self.supervisord_configuration_directory, '%s.conf' %
+          _getSupervisordConfigurationDirectory(self.instance_root), '%s.conf' %
           computer_partition_id),
         supervisord_socket=self.supervisord_socket,
         computer_partition=computer_partition,
@@ -670,6 +655,9 @@ class Slapgrid(object):
         buildout=self.buildout,
         logger=self.logger,
         retention_delay=retention_delay,
+        instance_min_free_space=self.instance_min_free_space,
+        instance_storage_home=self.instance_storage_home,
+        ipv4_global_network=self.ipv4_global_network,
       )
       computer_partition_state = computer_partition.getState()
 
@@ -737,8 +725,10 @@ class Slapgrid(object):
         except (NotFoundError, TypeError, NameError):
           software_url = None
         if computer_partition_state == COMPUTER_PARTITION_DESTROYED_STATE and \
-           os.listdir(computer_partition_path) == [] and \
            not software_url:
+          if os.listdir(computer_partition_path) != []:
+            self.logger.warning("Free partition %s contains file(s) in %s." % (
+                computer_partition.getId(), computer_partition_path))
           continue
 
         # Everything seems fine
@@ -1047,6 +1037,34 @@ class Slapgrid(object):
       self.logger.info('computer_partition_usage_list: %s - %s' %
                        (computer_partition_usage.usage, computer_partition_usage.getId()))
 
+    filename_delete_list = []
+    computer_report_dir = os.path.join(self.instance_root,
+                                 'var', 'xml_report', self.computer_id)
+
+    # The directory xml_report contain a number of files equal
+    # to the number of software instance running inside the same partition
+    if os.path.isdir(computer_report_dir):
+      filename_list = os.listdir(computer_report_dir)
+    else:
+      filename_list = []
+
+    for filename in filename_list:
+
+      file_path = os.path.join(computer_report_dir, filename)
+      if os.path.exists(file_path):
+        usage = open(file_path, 'r').read()
+
+      if self.validateXML(usage, computer_consumption_model):
+        self.logger.info('XML file generated by asXML is valid')
+        slap_computer_usage.reportUsage(usage)
+        filename_delete_list.append(filename)
+      else:
+        self.logger.info('XML file is invalid %s' % filename)
+
+    # After sending the aggregated file we remove all the valid xml reports
+    for filename in filename_delete_list:
+      os.remove(os.path.join(computer_report_dir, filename))
+
     # If there is, at least, one report
     if computer_partition_usage_list != []:
       try:
@@ -1072,6 +1090,7 @@ class Slapgrid(object):
 
     for computer_partition in computer_partition_list:
       if computer_partition.getState() == COMPUTER_PARTITION_DESTROYED_STATE:
+        destroyed = False
         try:
           computer_partition_id = computer_partition.getId()
           try:
@@ -1086,7 +1105,7 @@ class Slapgrid(object):
             instance_path=os.path.join(self.instance_root,
                 computer_partition.getId()),
             supervisord_partition_configuration_path=os.path.join(
-              self.supervisord_configuration_directory, '%s.conf' %
+              _getSupervisordConfigurationDirectory(self.instance_root), '%s.conf' %
               computer_partition_id),
             supervisord_socket=self.supervisord_socket,
             computer_partition=computer_partition,
@@ -1097,6 +1116,8 @@ class Slapgrid(object):
             certificate_repository_path=self.certificate_repository_path,
             buildout=self.buildout,
             logger=self.logger,
+            instance_storage_home=self.instance_storage_home,
+            ipv4_global_network=self.ipv4_global_network,
           )
           local_partition.stop()
           try:
