@@ -132,7 +132,8 @@ def getCleanEnvironment(logger, home_path='/tmp'):
   changed_env['HOME'] = env['HOME'] = home_path
   for k in sorted(changed_env.iterkeys()):
     logger.debug('Overridden %s = %r' % (k, changed_env[k]))
-  logger.debug('Removed from environment: %s' % ', '.join(sorted(removed_env)))
+  if removed_env:
+    logger.debug('Removed from environment: %s' % ', '.join(sorted(removed_env)))
   return env
 
 
@@ -248,7 +249,7 @@ def bootstrapBuildout(path, logger, buildout=None,
       logger.warning('Using old style bootstrap of included bootstrap file. '
         'Consider having zc.buildout available in search path.')
       invocation_list.append(pkg_resources.resource_filename(__name__,
-        'zc.buildout-bootstap.py'))
+        'zc.buildout-bootstrap.py'))
       invocation_list.extend(additional_buildout_parameter_list)
     else:
       # buildout is importable, so use this one
