@@ -95,11 +95,13 @@ def partitiondict2partition(partition):
     slap_partition._requested_state = partition['requested_state']
     slap_partition._parameter_dict = xml2dict(partition['xml'])
     address_list = []
+    full_address_list = []
     for address in execute_db('partition_network',
                               'SELECT * FROM %s WHERE partition_reference=? AND computer_reference=?',
                               [partition['reference'], partition['computer_reference']]):
       address_list.append((address['reference'], address['address']))
     slap_partition._parameter_dict['ip_list'] = address_list
+    slap_partition._parameter_dict['full_address_list'] = full_address_list
     slap_partition._parameter_dict['slap_software_type'] = \
         partition['software_type']
     if partition['slave_instance_list'] is not None:
@@ -229,19 +231,20 @@ def getFullComputerInformation():
 
 @app.route('/setComputerPartitionConnectionXml', methods=['POST'])
 def setComputerPartitionConnectionXml():
-  slave_reference = request.form['slave_reference'].encode()
+  slave_reference = request.form.get('slave_reference', None)
   computer_partition_id = request.form['computer_partition_id'].encode()
   computer_id = request.form['computer_id'].encode()
   connection_xml = request.form['connection_xml'].encode()
   connection_dict = xml_marshaller.xml_marshaller.loads(
                                             connection_xml)
   connection_xml = dict2xml(connection_dict)
-  if slave_reference == 'None':
+  if not slave_reference or slave_reference == 'None':
     query = 'UPDATE %s SET connection_xml=? WHERE reference=? AND computer_reference=?'
     argument_list = [connection_xml, computer_partition_id, computer_id]
     execute_db('partition', query, argument_list)
     return 'done'
   else:
+    slave_reference = slave_reference.encode()
     query = 'UPDATE %s SET connection_xml=? , hosted_by=? WHERE reference=?'
     argument_list = [connection_xml, computer_partition_id, slave_reference]
     execute_db('slave', query, argument_list)
@@ -495,7 +498,11 @@ def forwardRequestToExternalMaster(master_url, request_form):
   filter_kw['source_instance_id'] = partition_reference
   new_request_form['filter_xml'] = dumps(filter_kw)
 
-  partition = loads(slap._connection_helper.POST('/requestComputerPartition', new_request_form))
+  xml = slap._connection_helper.POST('/requestComputerPartition', data=new_request_form)
+  if type(xml) is unicode:
+    xml = str(xml)
+    xml.encode('utf-8')
+  partition = loads(xml)
 
   # XXX move to other end
   partition._master_url = master_url
@@ -634,7 +641,13 @@ def requestSlave(software_release, software_type, partition_reference, partition
     a(software_type)
   if 'instance_guid' in filter_kw:
     q += ' AND reference=?'
-    a(filter_kw['instance_guid'])
+    # instance_guid should be like: %s-%s % (requested_computer_id, partition_id)
+    # But code is convoluted here, so we check
+    instance_guid = filter_kw['instance_guid']
+    if instance_guid.startswith(requested_computer_id):
+      a(instance_guid[len(requested_computer_id) + 1:])
+    else:
+      a(instance_guid)
 
   partition = execute_db('partition', q, args, one=True)
   if partition is None:
@@ -719,6 +732,11 @@ def softwareInstanceRename():
 def getComputerPartitionStatus():
   return xml_marshaller.xml_marshaller.dumps('Not implemented.')
 
+@app.route('/getComputerPartitionCertificate', methods=['GET'])
+def getComputerPartitionCertificate():
+  # proxy does not use partition certificate, but client calls this.
+  return xml_marshaller.xml_marshaller.dumps({'certificate': '', 'key': ''})
+
 @app.route('/getSoftwareReleaseListFromSoftwareProduct', methods=['GET'])
 def getSoftwareReleaseListFromSoftwareProduct():
   software_product_reference = request.args.get('software_product_reference')
