@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# vim: set et sts=2:
 ##############################################################################
 #
 # Copyright (c) 2010, 2011, 2012 Vifib SARL and Contributors.
@@ -31,24 +32,32 @@ Simple, easy to (un)marshall classes for slap client/server communication
 """
 
 __all__ = ["slap", "ComputerPartition", "Computer", "SoftwareRelease",
-           "SoftwareProductCollection",
-           "Supply", "OpenOrder", "NotFoundError", "Unauthorized",
-           "ResourceNotReady", "ServerError"]
+           "SoftwareInstance", "SoftwareProductCollection",
+           "Supply", "OpenOrder", "NotFoundError",
+           "ResourceNotReady", "ServerError", "ConnectionError"]
 
-import httplib
+import json
 import logging
 import re
-import socket
-import ssl
-import urllib
 import urlparse
 from util import xml2dict
 
+import netaddr
 from xml.sax import saxutils
 import zope.interface
 from interface import slap as interface
 from xml_marshaller import xml_marshaller
 
+from uritemplate import expand
+
+import requests
+# silence messages like 'Unverified HTTPS request is being made'
+requests.packages.urllib3.disable_warnings()
+# silence messages like 'Starting connection' that are logged with INFO
+urllib3_logger = logging.getLogger('requests.packages.urllib3')
+urllib3_logger.setLevel(logging.WARNING)
+
+
 # XXX fallback_logger to be deprecated together with the old CLI entry points.
 fallback_logger = logging.getLogger(__name__)
 fallback_handler = logging.StreamHandler()
@@ -58,34 +67,13 @@ fallback_logger.addHandler(fallback_handler)
 
 DEFAULT_SOFTWARE_TYPE = 'RootSoftwareInstance'
 
-
-# httplib.HTTPSConnection with key verification
-class HTTPSConnectionCA(httplib.HTTPSConnection):
-  """Patched version of HTTPSConnection which verifies server certificate"""
-  def __init__(self, *args, **kwargs):
-    self.ca_file = kwargs.pop('ca_file')
-    if self.ca_file is None:
-      raise ValueError('ca_file is required argument.')
-    httplib.HTTPSConnection.__init__(self, *args, **kwargs)
-
-  def connect(self):
-    "Connect to a host on a given (SSL) port and verify its certificate."
-
-    sock = socket.create_connection((self.host, self.port),
-                                    self.timeout, self.source_address)
-    if self._tunnel_host:
-      self.sock = sock
-      self._tunnel()
-    self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
-        ca_certs=self.ca_file, cert_reqs=ssl.CERT_REQUIRED)
-
-
 class SlapDocument:
-  def __init__(self, connection_helper=None):
+  def __init__(self, connection_helper=None, hateoas_navigator=None):
     if connection_helper is not None:
       # Do not require connection_helper to be provided, but when it's not,
       # cause failures when accessing _connection_helper property.
       self._connection_helper = connection_helper
+      self._hateoas_navigator = hateoas_navigator
 
 
 class SlapRequester(SlapDocument):
@@ -94,12 +82,15 @@ class SlapRequester(SlapDocument):
   """
   def _requestComputerPartition(self, request_dict):
     try:
-      xml = self._connection_helper.POST('/requestComputerPartition', request_dict)
+      xml = self._connection_helper.POST('requestComputerPartition', data=request_dict)
     except ResourceNotReady:
       return ComputerPartition(
         request_dict=request_dict,
         connection_helper=self._connection_helper,
       )
+    if type(xml) is unicode:
+      xml = str(xml)
+      xml.encode('utf-8')
     software_instance = xml_marshaller.loads(xml)
     computer_partition = ComputerPartition(
       software_instance.slap_computer_id.encode('UTF-8'),
@@ -131,7 +122,8 @@ class SoftwareRelease(SlapDocument):
 
     XXX **kw args only kept for compatibility
     """
-    SlapDocument.__init__(self, kw.pop('connection_helper', None))
+    SlapDocument.__init__(self, kw.pop('connection_helper', None),
+                                kw.pop('hateoas_navigator', None))
     self._software_instance_list = []
     if software_release is not None:
       software_release = software_release.encode('UTF-8')
@@ -156,7 +148,7 @@ class SoftwareRelease(SlapDocument):
   def error(self, error_log, logger=None):
     try:
       # Does not follow interface
-      self._connection_helper.POST('/softwareReleaseError', {
+      self._connection_helper.POST('softwareReleaseError', data={
         'url': self.getURI(),
         'computer_id': self.getComputerId(),
         'error_log': error_log})
@@ -164,17 +156,17 @@ class SoftwareRelease(SlapDocument):
       (logger or fallback_logger).exception('')
 
   def available(self):
-    self._connection_helper.POST('/availableSoftwareRelease', {
+    self._connection_helper.POST('availableSoftwareRelease', data={
       'url': self.getURI(),
       'computer_id': self.getComputerId()})
 
   def building(self):
-    self._connection_helper.POST('/buildingSoftwareRelease', {
+    self._connection_helper.POST('buildingSoftwareRelease', data={
       'url': self.getURI(),
       'computer_id': self.getComputerId()})
 
   def destroyed(self):
-    self._connection_helper.POST('/destroyedSoftwareRelease', {
+    self._connection_helper.POST('destroyedSoftwareRelease', data={
       'url': self.getURI(),
       'computer_id': self.getComputerId()})
 
@@ -188,8 +180,9 @@ class SoftwareProductCollection(object):
   def __init__(self, logger, slap):
     self.logger = logger
     self.slap = slap
-    self.__getattr__ = self.get
-  def get(self, software_product):
+    self.get = self.__getattr__
+
+  def __getattr__(self, software_product):
       self.logger.info('Getting best Software Release corresponging to '
                        'this Software Product...')
       software_release_list = \
@@ -208,6 +201,7 @@ class SoftwareInstance(SlapDocument):
   """
   Contains Software Instance information
   """
+  zope.interface.implements(interface.ISoftwareInstance)
 
   def __init__(self, **kwargs):
     """
@@ -218,23 +212,21 @@ class SoftwareInstance(SlapDocument):
 
 
 """Exposed exceptions"""
-
-
-# XXX Why do we need to expose exceptions?
 class ResourceNotReady(Exception):
   zope.interface.implements(interface.IResourceNotReady)
 
-
 class ServerError(Exception):
   zope.interface.implements(interface.IServerError)
 
-
 class NotFoundError(Exception):
   zope.interface.implements(interface.INotFoundError)
 
+class AuthenticationError(Exception):
+  pass
 
-class Unauthorized(Exception):
-  zope.interface.implements(interface.IUnauthorized)
+
+class ConnectionError(Exception):
+  zope.interface.implements(interface.IConnectionError)
 
 
 class Supply(SlapDocument):
@@ -242,7 +234,7 @@ class Supply(SlapDocument):
 
   def supply(self, software_release, computer_guid=None, state='available'):
     try:
-      self._connection_helper.POST('/supplySupply', {
+      self._connection_helper.POST('supplySupply', data={
         'url': software_release,
         'computer_id': computer_guid,
         'state': state})
@@ -278,14 +270,27 @@ class OpenOrder(SlapRequester):
       request_dict['software_type'] = DEFAULT_SOFTWARE_TYPE
     return self._requestComputerPartition(request_dict)
 
+  def getInformation(self, partition_reference):
+    if not getattr(self, '_hateoas_navigator', None):
+      raise Exception('SlapOS Master Hateoas API required for this operation is not availble.')
+    raw_information = self._hateoas_navigator.getHostingSubscriptionRootSoftwareInstanceInformation(partition_reference)
+    software_instance = SoftwareInstance()
+    # XXX redefine SoftwareInstance to be more consistent
+    for key, value in raw_information.iteritems():
+      if key in ['_links']:
+        continue
+      setattr(software_instance, '_%s' % key, value)
+    setattr(software_instance, '_software_release_url', raw_information['_links']['software_release'])
+    return software_instance
+
   def requestComputer(self, computer_reference):
     """
     Requests a computer.
     """
-    xml = self._connection_helper.POST('/requestComputer',
-      {'computer_title': computer_reference})
+    xml = self._connection_helper.POST('requestComputer', data={'computer_title': computer_reference})
     computer = xml_marshaller.loads(xml)
     computer._connection_helper = self._connection_helper
+    computer._hateoas_navigator = self._hateoas_navigator
     return computer
 
 
@@ -313,8 +318,8 @@ def _syncComputerInformation(func):
 class Computer(SlapDocument):
   zope.interface.implements(interface.IComputer)
 
-  def __init__(self, computer_id, connection_helper=None):
-    SlapDocument.__init__(self, connection_helper)
+  def __init__(self, computer_id, connection_helper=None, hateoas_navigator=None):
+    SlapDocument.__init__(self, connection_helper, hateoas_navigator)
     self._computer_id = computer_id
 
   def __getinitargs__(self):
@@ -330,41 +335,41 @@ class Computer(SlapDocument):
     """
     for software_relase in self._software_release_list:
       software_relase._connection_helper = self._connection_helper
+      software_relase._hateoas_navigator = self._hateoas_navigator
     return self._software_release_list
 
   @_syncComputerInformation
   def getComputerPartitionList(self):
     for computer_partition in self._computer_partition_list:
       computer_partition._connection_helper = self._connection_helper
+      computer_partition._hateoas_navigator = self._hateoas_navigator
     return [x for x in self._computer_partition_list]
 
   def reportUsage(self, computer_usage):
     if computer_usage == "":
       return
-    self._connection_helper.POST('/useComputer', {
+    self._connection_helper.POST('useComputer', data={
       'computer_id': self._computer_id,
       'use_string': computer_usage})
 
   def updateConfiguration(self, xml):
-    return self._connection_helper.POST(
-        '/loadComputerConfigurationFromXML', {'xml': xml})
+    return self._connection_helper.POST('loadComputerConfigurationFromXML', data={'xml': xml})
 
   def bang(self, message):
-    self._connection_helper.POST('/computerBang', {
+    self._connection_helper.POST('computerBang', data={
       'computer_id': self._computer_id,
       'message': message})
 
   def getStatus(self):
-    xml = self._connection_helper.GET(
-        '/getComputerStatus?computer_id=%s' % self._computer_id)
+    xml = self._connection_helper.GET('getComputerStatus', params={'computer_id': self._computer_id})
     return xml_marshaller.loads(xml)
 
   def revokeCertificate(self):
-    self._connection_helper.POST('/revokeComputerCertificate', {
+    self._connection_helper.POST('revokeComputerCertificate', data={
       'computer_id': self._computer_id})
 
   def generateCertificate(self):
-    xml = self._connection_helper.POST('/generateComputerCertificate', {
+    xml = self._connection_helper.POST('generateComputerCertificate', data={
       'computer_id': self._computer_id})
     return xml_marshaller.loads(xml)
 
@@ -386,8 +391,9 @@ class ComputerPartition(SlapRequester):
   zope.interface.implements(interface.IComputerPartition)
 
   def __init__(self, computer_id=None, partition_id=None,
-               request_dict=None, connection_helper=None):
-    SlapDocument.__init__(self, connection_helper)
+               request_dict=None, connection_helper=None,
+               hateoas_navigator=None):
+    SlapDocument.__init__(self, connection_helper, hateoas_navigator)
     if request_dict is not None and (computer_id is not None or
         partition_id is not None):
       raise TypeError('request_dict conflicts with computer_id and '
@@ -436,36 +442,36 @@ class ComputerPartition(SlapRequester):
     return self._requestComputerPartition(request_dict)
 
   def building(self):
-    self._connection_helper.POST('/buildingComputerPartition', {
+    self._connection_helper.POST('buildingComputerPartition', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId()})
 
   def available(self):
-    self._connection_helper.POST('/availableComputerPartition', {
+    self._connection_helper.POST('availableComputerPartition', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId()})
 
   def destroyed(self):
-    self._connection_helper.POST('/destroyedComputerPartition', {
+    self._connection_helper.POST('destroyedComputerPartition', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId(),
       })
 
   def started(self):
-    self._connection_helper.POST('/startedComputerPartition', {
+    self._connection_helper.POST('startedComputerPartition', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId(),
       })
 
   def stopped(self):
-    self._connection_helper.POST('/stoppedComputerPartition', {
+    self._connection_helper.POST('stoppedComputerPartition', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId(),
       })
 
   def error(self, error_log, logger=None):
     try:
-      self._connection_helper.POST('/softwareInstanceError', {
+      self._connection_helper.POST('softwareInstanceError', data={
         'computer_id': self._computer_id,
         'computer_partition_id': self.getId(),
         'error_log': error_log})
@@ -473,7 +479,7 @@ class ComputerPartition(SlapRequester):
       (logger or fallback_logger).exception('')
 
   def bang(self, message):
-    self._connection_helper.POST('/softwareInstanceBang', {
+    self._connection_helper.POST('softwareInstanceBang', data={
       'computer_id': self._computer_id,
       'computer_partition_id': self.getId(),
       'message': message})
@@ -486,7 +492,26 @@ class ComputerPartition(SlapRequester):
             }
     if slave_reference:
       post_dict['slave_reference'] = slave_reference
-    self._connection_helper.POST('/softwareInstanceRename', post_dict)
+    self._connection_helper.POST('softwareInstanceRename', data=post_dict)
+
+  def getInformation(self, partition_reference):
+    """
+    Return all needed informations about an existing Computer Partition
+    in the Instance tree of the current Computer Partition.
+    """
+    if not getattr(self, '_hateoas_navigator', None):
+      raise Exception('SlapOS Master Hateoas API required for this operation is not availble.')
+
+    raw_information = self._hateoas_navigator.getRelatedInstanceInformation(partition_reference)
+    software_instance = SoftwareInstance()
+    # XXX redefine SoftwareInstance to be more consistent
+    for key, value in raw_information.iteritems():
+      if key in ['_links']:
+        continue
+      setattr(software_instance, '_%s' % key, value)
+    setattr(software_instance, '_software_release_url', raw_information['_links']['software_release'])
+    return software_instance
+
 
   def getId(self):
     if not getattr(self, '_partition_id', None):
@@ -524,10 +549,10 @@ class ComputerPartition(SlapRequester):
     connection_dict = getattr(self, '_connection_dict', None)
     if connection_dict is None:
       # XXX Backward compatibility for older slapproxy (<= 1.0.0)
-      connection_dict = xml2dict(getattr(self, 'connection_xml', '')) 
+      connection_dict = xml2dict(getattr(self, 'connection_xml', ''))
 
     return connection_dict or {}
-      
+
   def getSoftwareRelease(self):
     """
     Returns the software release associate to the computer partition.
@@ -540,7 +565,7 @@ class ComputerPartition(SlapRequester):
 
   def setConnectionDict(self, connection_dict, slave_reference=None):
     if self.getConnectionParameterDict() != connection_dict:
-      self._connection_helper.POST('/setComputerPartitionConnectionXml', {
+      self._connection_helper.POST('setComputerPartitionConnectionXml', data={
           'computer_id': self._computer_id,
           'computer_partition_id': self._partition_id,
           'connection_xml': xml_marshaller.dumps(connection_dict),
@@ -565,39 +590,60 @@ class ComputerPartition(SlapRequester):
     self.usage = usage_log
 
   def getCertificate(self):
-    xml = self._connection_helper.GET(
-        '/getComputerPartitionCertificate?computer_id=%s&'
-        'computer_partition_id=%s' % (self._computer_id, self._partition_id))
+    xml = self._connection_helper.GET('getComputerPartitionCertificate',
+            params={
+                'computer_id': self._computer_id,
+                'computer_partition_id': self._partition_id,
+                }
+            )
     return xml_marshaller.loads(xml)
 
   def getStatus(self):
-    xml = self._connection_helper.GET(
-        '/getComputerPartitionStatus?computer_id=%s&'
-        'computer_partition_id=%s' % (self._computer_id, self._partition_id))
+    xml = self._connection_helper.GET('getComputerPartitionStatus',
+            params={
+                'computer_id': self._computer_id,
+                'computer_partition_id': self._partition_id,
+                }
+            )
     return xml_marshaller.loads(xml)
 
+def _addIpv6Brackets(url):
+  # if master_url contains an ipv6 without bracket, add it
+  # Note that this is mostly to limit specific issues with
+  # backward compatiblity, not to ensure generic detection.
+  api_scheme, api_netloc, api_path, api_query, api_fragment = urlparse.urlsplit(url)
+  try:
+    ip = netaddr.IPAddress(api_netloc)
+    port = None
+  except netaddr.AddrFormatError:
+    try:
+      ip = netaddr.IPAddress(':'.join(api_netloc.split(':')[:-1]))
+      port = api_netloc.split(':')[-1]
+    except netaddr.AddrFormatError:
+      ip = port = None
+  if ip and ip.version == 6:
+    api_netloc = '[%s]' % ip
+    if port:
+      api_netloc = '%s:%s' % (api_netloc, port)
+    url = urlparse.urlunsplit((api_scheme, api_netloc, api_path, api_query, api_fragment))
+  return url
 
 class ConnectionHelper:
-  error_message_timeout = "\nThe connection timed out. Please try again later."
-  error_message_connect_fail = "Couldn't connect to the server. Please " \
-      "double check given master-url argument, and make sure that IPv6 is " \
-      "enabled on your machine and that the server is available. The " \
-      "original error was: "
-  ssl_error_message_connect_fail = "\nCouldn't authenticate computer. Please "\
-      "check that certificate and key exist and are valid. "
-
-  def __init__(self, connection_wrapper, host, path, key_file=None,
+  def __init__(self, master_url, key_file=None,
                cert_file=None, master_ca_file=None, timeout=None):
-    self.connection_wrapper = connection_wrapper
-    self.host = host
-    self.path = path
+    master_url = _addIpv6Brackets(master_url)
+    if master_url.endswith('/'):
+        self.slapgrid_uri = master_url
+    else:
+        # add a slash or the last path segment will be ignored by urljoin
+        self.slapgrid_uri = master_url + '/'
     self.key_file = key_file
     self.cert_file = cert_file
     self.master_ca_file = master_ca_file
     self.timeout = timeout
 
   def getComputerInformation(self, computer_id):
-    xml = self.GET('/getComputerInformation?computer_id=%s' % computer_id)
+    xml = self.GET('getComputerInformation', params={'computer_id': computer_id})
     return xml_marshaller.loads(xml)
 
   def getFullComputerInformation(self, computer_id):
@@ -605,123 +651,131 @@ class ConnectionHelper:
     Retrieve from SlapOS Master Computer instance containing all needed
     informations (Software Releases, Computer Partitions, ...).
     """
-    method = '/getFullComputerInformation?computer_id=%s' % computer_id
+    path = 'getFullComputerInformation'
+    params = {'computer_id': computer_id}
     if not computer_id:
       # XXX-Cedric: should raise something smarter than "NotFound".
-      raise NotFoundError(method)
+      raise NotFoundError('%r %r' % (path, params))
     try:
-      xml = self.GET(method)
+      xml = self.GET(path, params=params)
     except NotFoundError:
       # XXX: This is a ugly way to keep backward compatibility,
       # We should stablise slap library soon.
-      xml = self.GET('/getComputerInformation?computer_id=%s' % computer_id)
+      xml = self.GET('getComputerInformation', params=params)
 
+    if type(xml) is unicode:
+      xml = str(xml)
+      xml.encode('utf-8')
     return xml_marshaller.loads(xml)
 
-  def connect(self):
-    connection_dict = {
-            'host': self.host
-            }
-    if self.key_file and self.cert_file:
-      connection_dict['key_file'] = self.key_file
-      connection_dict['cert_file'] = self.cert_file
-    if self.master_ca_file:
-      connection_dict['ca_file'] = self.master_ca_file
-    self.connection = self.connection_wrapper(**connection_dict)
+  def do_request(self, method, path, params=None, data=None, headers=None):
+    url = urlparse.urljoin(self.slapgrid_uri, path)
+    if headers is None:
+      headers = {}
+    headers.setdefault('Accept', '*/*')
+    if path.startswith('/'):
+      path = path[1:]
+#      raise ValueError('method path should be relative: %s' % path)
 
-  def GET(self, path):
     try:
-      default_timeout = socket.getdefaulttimeout()
-      socket.setdefaulttimeout(self.timeout)
-      try:
-        self.connect()
-        self.connection.request('GET', self.path + path)
-        response = self.connection.getresponse()
-      # If ssl error : may come from bad configuration
-      except ssl.SSLError as exc:
-        if exc.message == 'The read operation timed out':
-          raise socket.error(str(exc) + self.error_message_timeout)
-        raise ssl.SSLError(str(exc) + self.ssl_error_message_connect_fail)
-      except socket.error as exc:
-        if exc.message == 'timed out':
-          raise socket.error(str(exc) + self.error_message_timeout)
-        raise socket.error(self.error_message_connect_fail + str(exc))
+      if url.startswith('https'):
+        cert = (self.cert_file, self.key_file)
+      else:
+        cert = None
 
-      # check self.response.status and raise exception early
-      if response.status == httplib.REQUEST_TIMEOUT:
-        # resource is not ready
+      # XXX TODO: handle host cert verify
+
+      # Old behavior was to pass empty parameters as "None" value.
+      # Behavior kept for compatibility with old slapproxies (< v1.3.3).
+      # Can be removed when old slapproxies are no longer in use.
+      if data:
+        for k, v in data.iteritems():
+          if v is None:
+            data[k] = 'None'
+
+      req = method(url=url,
+                   params=params,
+                   cert=cert,
+                   verify=False,
+                   data=data,
+                   headers=headers,
+                   timeout=self.timeout)
+      req.raise_for_status()
+
+    except (requests.Timeout, requests.ConnectionError) as exc:
+      raise ConnectionError("Couldn't connect to the server. Please "
+                            "double check given master-url argument, and make sure that IPv6 is "
+                            "enabled on your machine and that the server is available. The "
+                            "original error was:\n%s" % exc)
+    except requests.HTTPError as exc:
+      if exc.response.status_code == requests.status_codes.codes.not_found:
+        msg = url
+        if params:
+            msg += ' - %s' % params
+        raise NotFoundError(msg)
+      elif exc.response.status_code == requests.status_codes.codes.request_timeout:
+        # this is explicitly returned by SlapOS master, and does not really mean timeout
         raise ResourceNotReady(path)
-      elif response.status == httplib.NOT_FOUND:
-        raise NotFoundError(path)
-      elif response.status == httplib.FORBIDDEN:
-        raise Unauthorized(path)
-      elif response.status != httplib.OK:
-        message = parsed_error_message(response.status,
-                                       response.read(),
-                                       path)
-        raise ServerError(message)
-    finally:
-      socket.setdefaulttimeout(default_timeout)
+        # XXX TODO test request timeout and resource not found
+      else:
+        # we don't know how or don't want to handle these (including Unauthorized)
+        req.raise_for_status()
+    except requests.exceptions.SSLError as exc:
+      raise AuthenticationError("%s\nCouldn't authenticate computer. Please "
+                                "check that certificate and key exist and are valid." % exc)
 
-    return response.read()
+#    XXX TODO parse server messages for client configure and node register
+#    elif response.status != httplib.OK:
+#      message = parsed_error_message(response.status,
+#                                     response.read(),
+#                                     path)
+#      raise ServerError(message)
 
-  def POST(self, path, parameter_dict,
+    return req
+
+  def GET(self, path, params=None, headers=None):
+    req = self.do_request(requests.get,
+                          path=path,
+                          params=params,
+                          headers=headers)
+    return req.text.encode('utf-8')
+
+  def POST(self, path, params=None, data=None,
            content_type='application/x-www-form-urlencoded'):
-    try:
-      default_timeout = socket.getdefaulttimeout()
-      socket.setdefaulttimeout(self.timeout)
-      try:
-        self.connect()
-        header_dict = {'Content-type': content_type}
-        self.connection.request("POST", self.path + path,
-            urllib.urlencode(parameter_dict), header_dict)
-      # If ssl error : must come from bad configuration
-      except ssl.SSLError as exc:
-        raise ssl.SSLError(self.ssl_error_message_connect_fail + str(exc))
-      except socket.error as exc:
-        raise socket.error(self.error_message_connect_fail + str(exc))
-
-      response = self.connection.getresponse()
-      # check self.response.status and raise exception early
-      if response.status == httplib.REQUEST_TIMEOUT:
-        # resource is not ready
-        raise ResourceNotReady("%s - %s" % (path, parameter_dict))
-      elif response.status == httplib.NOT_FOUND:
-        raise NotFoundError("%s - %s" % (path, parameter_dict))
-      elif response.status == httplib.FORBIDDEN:
-        raise Unauthorized("%s - %s" % (path, parameter_dict))
-      elif response.status != httplib.OK:
-        message = parsed_error_message(response.status,
-                                       response.read(),
-                                       path)
-        raise ServerError(message)
-    finally:
-      socket.setdefaulttimeout(default_timeout)
-
-    return response.read()
+    req = self.do_request(requests.post,
+                          path=path,
+                          params=params,
+                          data=data,
+                          headers={'Content-type': content_type})
+    return req.text.encode('utf-8')
 
 
 class slap:
   zope.interface.implements(interface.slap)
 
-  def initializeConnection(self, slapgrid_uri, key_file=None, cert_file=None,
-                           master_ca_file=None, timeout=60):
-    scheme, netloc, path, query, fragment = urlparse.urlsplit(slapgrid_uri)
-    if not (query == '' and fragment == ''):
-      raise AttributeError('Passed URL %r issue: not parseable' % slapgrid_uri)
+  def initializeConnection(self, slapgrid_uri,
+                           key_file=None, cert_file=None,
+                           master_ca_file=None,
+                           timeout=60,
+                           slapgrid_rest_uri=None):
+    if master_ca_file:
+      raise NotImplementedError('Master certificate not verified in this version: %s' % master_ca_file)
 
-    if scheme == 'http':
-      connection_wrapper = httplib.HTTPConnection
-    elif scheme == 'https':
-      if master_ca_file is not None:
-        connection_wrapper = HTTPSConnectionCA
-      else:
-        connection_wrapper = httplib.HTTPSConnection
+    self._connection_helper = ConnectionHelper(slapgrid_uri, key_file, cert_file, master_ca_file, timeout)
+
+    if not slapgrid_rest_uri:
+      try:
+        slapgrid_rest_uri = self._connection_helper.GET('getHateoasUrl')
+      except:
+        pass
+    if slapgrid_rest_uri:
+      self._hateoas_navigator = SlapHateoasNavigator(
+          slapgrid_rest_uri,
+          key_file, cert_file,
+          master_ca_file, timeout
+      )
     else:
-      raise AttributeError('Passed URL %r issue: there is no support '
-                           'for %r protocol' % (slapgrid_uri, scheme))
-    self._connection_helper = ConnectionHelper(connection_wrapper,
-          netloc, path, key_file, cert_file, master_ca_file, timeout)
+      self._hateoas_navigator = None
 
   # XXX-Cedric: this method is never used and thus should be removed.
   def registerSoftwareRelease(self, software_release):
@@ -730,7 +784,8 @@ class slap:
     returns SoftwareRelease class object
     """
     return SoftwareRelease(software_release=software_release,
-      connection_helper=self._connection_helper
+      connection_helper=self._connection_helper,
+      hateoas_navigator=self._hateoas_navigator
     )
 
   def registerComputer(self, computer_guid):
@@ -738,7 +793,10 @@ class slap:
     Registers connected representation of computer and
     returns Computer class object
     """
-    return Computer(computer_guid, connection_helper=self._connection_helper)
+    return Computer(computer_guid,
+      connection_helper=self._connection_helper,
+      hateoas_navigator=self._hateoas_navigator
+    )
 
   def registerComputerPartition(self, computer_guid, partition_id):
     """
@@ -749,35 +807,206 @@ class slap:
       # XXX-Cedric: should raise something smarter than NotFound
       raise NotFoundError
 
-    xml = self._connection_helper.GET('/registerComputerPartition?' \
-        'computer_reference=%s&computer_partition_reference=%s' % (
-          computer_guid, partition_id))
+    xml = self._connection_helper.GET('registerComputerPartition',
+            params = {
+                'computer_reference': computer_guid,
+                'computer_partition_reference': partition_id,
+                }
+            )
+    if type(xml) is unicode:
+      xml = str(xml)
+      xml.encode('utf-8')
     result = xml_marshaller.loads(xml)
     # XXX: dirty hack to make computer partition usable. xml_marshaller is too
     # low-level for our needs here.
     result._connection_helper = self._connection_helper
+    result._hateoas_navigator = self._hateoas_navigator
     return result
 
   def registerOpenOrder(self):
-    return OpenOrder(connection_helper=self._connection_helper)
+    return OpenOrder(
+      connection_helper=self._connection_helper,
+      hateoas_navigator=self._hateoas_navigator
+  )
 
   def registerSupply(self):
-    return Supply(connection_helper=self._connection_helper)
+    return Supply(
+      connection_helper=self._connection_helper,
+      hateoas_navigator=self._hateoas_navigator
+  )
 
   def getSoftwareReleaseListFromSoftwareProduct(self,
       software_product_reference=None, software_release_url=None):
-    url = '/getSoftwareReleaseListFromSoftwareProduct?'
+    url = 'getSoftwareReleaseListFromSoftwareProduct'
+    params = {}
     if software_product_reference:
       if software_release_url is not None:
         raise AttributeError('Both software_product_reference and '
                              'software_release_url parameters are specified.')
-      url += 'software_product_reference=%s' % software_product_reference
+      params['software_product_reference'] = software_product_reference
     else:
       if software_release_url is None:
         raise AttributeError('None of software_product_reference and '
                              'software_release_url parameters are specified.')
-      url += 'software_release_url=%s' % software_release_url
+      params['software_release_url'] = software_release_url
 
-    result = xml_marshaller.loads(self._connection_helper.GET(url))
+    xml = self._connection_helper.GET(url, params=params)
+    if type(xml) is unicode:
+      xml = str(xml)
+      xml.encode('utf-8')
+    result = xml_marshaller.loads(xml)
     assert(type(result) == list)
     return result
+
+  def getOpenOrderDict(self):
+    if not getattr(self, '_hateoas_navigator', None):
+      raise Exception('SlapOS Master Hateoas API required for this operation is not availble.')
+    return self._hateoas_navigator.getHostingSubscriptionDict()
+
+class HateoasNavigator(object):
+  """
+  Navigator for HATEOAS-style APIs.
+  Inspired by
+  https://git.erp5.org/gitweb/jio.git/blob/HEAD:/src/jio.storage/erp5storage.js
+  """
+  # XXX: needs to be designed for real. For now, just a non-maintainable prototype.
+  # XXX: export to a standalone library, independant from slap.
+  def __init__(self, slapgrid_uri,
+               key_file=None, cert_file=None,
+               master_ca_file=None, timeout=60):
+    self.slapos_master_hateoas_uri = slapgrid_uri
+    self.key_file = key_file
+    self.cert_file = cert_file
+    self.master_ca_file = master_ca_file
+    self.timeout = timeout
+
+  def GET(self, uri, headers=None):
+    connection_helper = ConnectionHelper(
+        uri, self.key_file, self.cert_file, self.master_ca_file, self.timeout)
+    return connection_helper.GET(uri, headers=headers)
+
+  def hateoasGetLinkFromLinks(self, links, title):
+    if type(links) == dict:
+      if links.get('title') == title:
+        return links['href']
+      raise NotFoundError('Action %s not found.' % title)
+    for action in links:
+      if action.get('title') == title:
+        return action['href']
+    else:
+      raise NotFoundError('Action %s not found.' % title)
+
+  def getRelativeUrlFromUrn(self, urn):
+    urn_schema = 'urn:jio:get:'
+    try:
+      _, url = urn.split(urn_schema)
+    except ValueError:
+      return
+    return str(url)
+
+  def getSiteDocument(self, url, headers=None):
+    result = self.GET(url, headers)
+    return json.loads(result)
+
+  def getRootDocument(self):
+    # XXX what about cache?
+    cached_root_document = getattr(self, 'root_document', None)
+    if cached_root_document:
+      return cached_root_document
+    self.root_document = self.getSiteDocument(
+        self.slapos_master_hateoas_uri,
+        headers={'Cache-Control': 'no-cache'}
+    )
+    return self.root_document
+
+  def getDocumentAndHateoas(self, relative_url, view='view'):
+    site_document = self.getRootDocument()
+    return expand(
+        site_document['_links']['traverse']['href'],
+        dict(relative_url=relative_url, view=view)
+    )
+
+  def getMeDocument(self):
+    person_relative_url = self.getRelativeUrlFromUrn(
+        self.getRootDocument()['_links']['me']['href'])
+    person_url = self.getDocumentAndHateoas(person_relative_url)
+    return json.loads(self.GET(person_url))
+
+class SlapHateoasNavigator(HateoasNavigator):
+  def _hateoas_getHostingSubscriptionDict(self):
+    action_object_slap_list = self.getMeDocument()['_links']['action_object_slap']
+    for action in action_object_slap_list:
+      if action.get('title') == 'getHateoasHostingSubscriptionList':
+        getter_link = action['href']
+        break
+    else:
+      raise Exception('Hosting subscription not found.')
+    result = self.GET(getter_link)
+    return json.loads(result)['_links']['content']
+
+  # XXX rename me to blablaUrl(self)
+  def _hateoas_getRelatedHostingSubscription(self):
+    action_object_slap_list = self.getMeDocument()['_links']['action_object_slap']
+    getter_link = self.hateoasGetLinkFromLinks(action_object_slap_list, 'getHateoasRelatedHostingSubscription')
+    result = self.GET(getter_link)
+    return json.loads(result)['_links']['action_object_jump']['href']
+
+  def _hateoasGetInformation(self, url):
+    result = self.GET(url)
+    result = json.loads(result)
+    object_link = self.hateoasGetLinkFromLinks(
+      result['_links']['action_object_slap'],
+      'getHateoasInformation'
+    )
+    result = self.GET(object_link)
+    return json.loads(result)
+
+  def getHateoasInstanceList(self, hosting_subscription_url):
+    hosting_subscription = json.loads(self.GET(hosting_subscription_url))
+    instance_list_url = self.hateoasGetLinkFromLinks(hosting_subscription['_links']['action_object_slap'], 'getHateoasInstanceList')
+    instance_list = json.loads(self.GET(instance_list_url))
+    return instance_list['_links']['content']
+
+  def getHostingSubscriptionDict(self):
+    hosting_subscription_link_list = self._hateoas_getHostingSubscriptionDict()
+    hosting_subscription_dict = {}
+    for hosting_subscription_link in hosting_subscription_link_list:
+      raw_information = self.getHostingSubscriptionRootSoftwareInstanceInformation(hosting_subscription_link['title'])
+      software_instance = SoftwareInstance()
+      # XXX redefine SoftwareInstance to be more consistent
+      for key, value in raw_information.iteritems():
+        if key in ['_links']:
+          continue
+        setattr(software_instance, '_%s' % key, value)
+      setattr(software_instance, '_software_release_url', raw_information['_links']['software_release'])
+      hosting_subscription_dict[software_instance._title] = software_instance
+
+    return hosting_subscription_dict
+
+  def getHostingSubscriptionRootSoftwareInstanceInformation(self, reference):
+    hosting_subscription_list = self._hateoas_getHostingSubscriptionDict()
+    for hosting_subscription in hosting_subscription_list:
+      if hosting_subscription.get('title') == reference:
+        hosting_subscription_url = hosting_subscription['href']
+        break
+    else:
+      raise NotFoundError('This document does not exist.')
+
+    hosting_subscription = json.loads(self.GET(hosting_subscription_url))
+
+    software_instance_url = self.hateoasGetLinkFromLinks(
+        hosting_subscription['_links']['action_object_slap'],
+        'getHateoasRootInstance'
+    )
+    response = self.GET(software_instance_url)
+    response = json.loads(response)
+    software_instance_url = response['_links']['content'][0]['href']
+    return self._hateoasGetInformation(software_instance_url)
+
+  def getRelatedInstanceInformation(self, reference):
+    related_hosting_subscription_url = self._hateoas_getRelatedHostingSubscription()
+    instance_list = self.getHateoasInstanceList(related_hosting_subscription_url)
+    instance_url = self.hateoasGetLinkFromLinks(instance_list, reference)
+    instance = self._hateoasGetInformation(instance_url)
+    return instance
+
